/*
	parloop
	Cpyright Risto Holopainen 2021
	
	parametric sequencer / looper
	
	 This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/


#include <vector>
#include <cmath>
#include <cstdio>

using std::vector;


void init_p(vector<double> &p, unsigned int sz)
{
	double x = 1/2.;
	for(unsigned int n=0; n<sz; n++)
		{
		x = 1.0 - sin(M_PI*x);
		p.push_back(x);
		}
}



class looper
{
	public:
		looper(unsigned int tapelength);
		~looper();
		double read();
		double read(uint t) const;
		void overwrite(double x);	
		void set_loop(uint beg, uint end);
		vector<double> borrow_tape() const;	// send copy of tape
		void replace_tape(const vector<double> &external);

	private:
		vector<double> tape;
		unsigned int TL, t, wp, begin, end;
};


looper::looper(unsigned int len)
{
	TL = len;
	init_p(tape, TL);
	t = 0;
	begin = 0;
	end = TL-1;
	wp = 0;		// write pointer
}


looper::~looper()
{
}


double looper::read()
{
	double x = tape[t];	
	if(++t >= end)
		t = begin;
	
	return x;
}

double looper::read(uint t) const
{
	uint k = t;
	if(t < 0)
		k = 0;
	else if(t >= TL)
		t = TL;

	return tape[k];
}

void looper::overwrite(double x)
{
	tape[wp] = x;
	if(++wp >= TL)
		wp = 0;
}


void looper::set_loop(uint begp, uint endp)
{
	if(begp >= 0 && begp < endp && begp < TL)
		begin = begp;
	else
		begin = 0;
	if(endp < TL && endp > begin)
		end = endp;
	else
		end = TL;
}


vector<double> looper::borrow_tape() const	// send copy of tape
{
	return tape;
}

void looper::replace_tape(const vector<double> &external)
{
	TL = external.size();
	if(end > TL) end = TL;
	if(begin >= end)
		begin = 0;

	tape = external;
}
