
/*
	folie.h
	<c> Risto Holopainen 2021
	
	phase accumulator -> LFO, time counter
	
	 This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/


#ifndef FOLIE_H
#define FOLIE_H

#include <cmath>

struct loco
{
	double u=0.0;
	double v=0.0;
	double q=0.0;
	unsigned char count;
};


class il_est_fou
{
	public:
		il_est_fou(unsigned char stop);
		loco run(double x);
	
	private:
		unsigned char u, stop;
		double alpha;
		double theta;
		double loss;
		double leak;
		double decay;
		double twopi;
};


il_est_fou::il_est_fou(unsigned char SS)
{
	stop = SS;
	alpha = 0.0;
	theta = 0.0;
	u = 0;
	leak = 1 - 2E-06;
	decay = 1 - 5E-05;
	loss = leak;
	twopi = 2*M_PI;
}


loco il_est_fou::run(double xi)
{
	theta += xi;
	alpha += xi;
	alpha *= loss;

	if(alpha > twopi)
		{
		++u;
		alpha = 0;
		if(u == stop)
			loss = decay;
		}
	
	loco LFO;
	
	LFO.u = sin(theta);
	LFO.v = cos(theta);
	LFO.q = .5 * (1 + sin(alpha));
	LFO.count = u;
	
	return LFO;
}

#endif

