/*
	d_tuning.h
	<c> Risto Holopainen 2021

	helper functions for The Wastebasket of Timbre
	generate scale: pure+tempered tritone

	This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/


#include <cstdio>
#include <cmath>


double cent(double r)
{
	double cc = 1200 * log(r) / log(2);
	return cc;
}


class d_tuning
{
	public:
		enum tonality {G, Ciss, Db};
		d_tuning();
		double frq(int pc, int oct=0, tonality T=G);
	
	private:
		double pure_g[12];
		double p_ciss[12];
		double ref;		
		void init_pure();
		void init_ciss();
};


d_tuning::d_tuning()
{
	init_pure();	// init pure_g
	init_ciss();
	ref = 396;
}



void d_tuning::init_pure()
{
	const double pc_g[12] = { 1, 16./15, 10/9., 6/5., 5/4., 4/3., 
				sqrt(2.0), 3/2., 8/5., 5/3., 16/9., 15/8.};

	for(int i=0; i<12; i++)
		pure_g[i] = pc_g[i];
/*
	
	printf("\n---------pure:------------\n");
	for(int i=0; i<12; i++)
		printf("˙%2d: \t%.5f \t= %+.1f ¢\n", 
			i, pc_g[i], cent(pc_g[i]));
*/
}




void d_tuning::init_ciss()
{
	// g, g#, a, ...c#, ..., f#
	const double ci = sqrt(2.0);
	const double Q = 3/2.;
	double p = ci, r = ci;
	p_ciss[0] = 1;
	p_ciss[6] = ci;

	for(int j=1, m=6, n=6; j<6; j++)
		{
		m = (m+7)%12;	// c# --> +fifths
		n = (n+5)%12;	// c# <<- -fifths
		p *= Q;
		r /= Q;
		while(p > 2)
			p /= 2.;
		while(r < 1)
			r *= 2.;

		p_ciss[m] = p;
		p_ciss[n] = r;
		}
	/*
	printf("\n");
	for(int i=0; i<12; i++)
		printf("˙%2d: \t%.5f \t= %+.1f ¢\n", 
			i, p_ciss[i], cent(p_ciss[i]));
	*/
}

double d_tuning::frq(int pc, int oct, tonality T)
{
	double ff = ref;
	int Note = pc%12;
	
	if (T == G)
		{
		ff *= pure_g[Note];
		}
	else	// T=Db, T=C#
		{
		ff *= p_ciss[Note];	
		}
	double s = (oct < 0) ? 1.0 / (1 << (-oct)) : (1 << oct) ;
	return s*ff;
}





